//<c>   Copyright 1980-1999 by Gerry J. Danen; all rights reserved.

//<t>   Format a Number according to a "Picture" Clause

#include"gd_tools.h"


INT16   PicAmount( INT32 number, UCHAR dest[], UCHAR picture[] )
/*
        Format a number of type INT32 into a string according to formatting
        rules contained in a picture, somewhat like COBOL.

        "picture" can contain:
                +   insert sign, '+' or '-' (must be first or last in string)
                -   insert sign, ' ' or '-' (must be first or last in string)
                .   decimal:  number is automatically scaled to decimal point
                ,   comma for separating groups of digits (cannot occur after
                    decimal point)
                9   digit 0 - 9, or leading zeros
                Z   digit 0 - 9, or leading spaces
                _   same as "Z"

        "picture" can be passed as a literal, since it is NOT changed anyway.

        Examples of legal "pictures":
        "ZZZZZZZZZZ.ZZ+", "ZZZZZZZZZZ.ZZ-", "9999999999.99+", "ZZ,ZZZ,ZZ9.99-",
        "ZZ,ZZZ,ZZ9.99+", "-ZZ,ZZZ,ZZ9.99", "+ZZ,ZZZ,ZZ9.99", "ZZZ,ZZZ,ZZ9.99",
        "Z,ZZ9.99", "9,999.99", "99999.99", "ZZZZZ.99", "ZZZZZ.ZZ", "9.999999",
        "ZZZZZZZZ", "99999999", "99999ZZZ"

        CAVEAT: The size of dest[] MUST be at least 2 bytes LONGER than the
                given picture.  For "ZZ9", dest must be at least [5] chars.
*/
{
    UCHAR       buf[15], sign, ch;
    UCHAR       *p_begin, *p_end, *pic_end, *ptr;
    FLAG        negative=FALSE, sign_leading=TRUE, sign_trailing=FALSE;
    INT16       ct_plus=0, ct_minus=0, ct_dot=0, ct_comma=0, ct_9=0, ct_Z=0, ct_other=0;

    dest[0] = EOS ;                     //  set destination to empty
    if ( strlen(picture) > 14 )         //  maximum size of picture
        return( ERROR );                //  too big
    if ( strlen(picture) < 2 )          //  minimum size of picture
        return( ERROR );                //  too small
    st_cpy( dest, picture, 1+strlen(picture) ); // copy of picture
    ptr = dest;
    while ( *ptr )
    {
        if ( *ptr == '_' )              //  replace all underscores with Z
            *ptr = 'Z';
        ptr++;
    }
    _strupr( dest );                    //  make uppercase

    //  Analyze picture string

    pic_end = dest + strlen(dest) - 1;  //  end of destination string

    //  Count how many times each character occurs
    ptr = dest;
    while ( *ptr )
    {
        ch = *ptr;
        switch ( ch )
        {
            case '+':   ct_plus++;  break;
            case '-':   ct_minus++; break;
            case '.':   ct_dot++;   break;
            case ',':   ct_comma++; break;
            case '9':   ct_9++;     break;
            case 'Z':   if ( ct_dot || ct_9 )   //  Z after . or 9 becomes 9
                        {
                            *ptr = '9';
                            ct_9++;
                        }
                        else
                        {
                            ct_Z++;
                        }
                        break;
            default :   ct_other++; break;
        }
        ptr++;
    }

    if ( ct_other != 0 )
        return( ERROR );                //  illegal characters in picture
    if ( (ct_plus+ct_minus) > 1 )
        return( ERROR );                //  too many sign characters
    if ( ct_dot > 1 )
        return( ERROR );                //  too many dots (decimal)

    //                                      Where do we want the sign?
    if ( (*pic_end == '-') || (*pic_end == '+') )
    {
        sign_leading  = FALSE;
        sign_trailing = TRUE ;
    }

    //                                      Put number in string buffer
    p_begin = _ltoa( number, buf, 10 );
    if ( *p_begin == '-' )
    {
        negative=TRUE;
        p_begin++;
    }
    p_end   = p_begin + strlen(p_begin) - 1;

    if ( sign_trailing )                //  trailing sign
    {
        sign = *pic_end;
    }
    else                                //  leading sign
    {
        sign = *dest;
        ptr  = dest;
        if ( (sign == '-') || (sign == '+') )
        {
            if ( *(++ptr) == ',' )
            {
                ptr++;
                if ( (*ptr != '9') && (*ptr != 'Z') )
                {
                    StrFill( dest, '*', strlen(dest) );
                    return( ERROR );    //  s/b 9 or Z following ","
                }
            }
            *dest = *ptr;
        }
    }
    if ( (sign != '-') && (sign != '+') )
        sign = '-';
    if ( negative )
        sign = '-';
    if ( sign_trailing )                //  insert trailing sign
    {
        if ( negative )
        {
            *pic_end = '-';
        }
        else
        {
            if ( sign == '+' )
                *pic_end = '+';
            else
                *pic_end = ' ';
        }
        pic_end--;
    }
    while ( (p_end NOT_LESS p_begin) && (pic_end NOT_LESS dest) )
    {
        if ( (*pic_end != 'Z') && (*pic_end != '9') )
            pic_end--;
        else
        {
            *(pic_end--) = *(p_end--);
        }
    }

    if ( p_end NOT_LESS p_begin )       //  picture too small
    {
        StrFill( dest, '*', strlen(dest) );
        return( ERROR );
    }

    if ( pic_end NOT_LESS dest )        //  leading spaces or zeros
    {
        while ( pic_end NOT_LESS dest )
        {
            if ( *pic_end == 'Z' )
                *pic_end = ' ';
            else
                if ( *pic_end == '9' )
                    *pic_end = '0';
            pic_end--;
        }
    }
    //                                      Check for commas before spaces
    ptr = dest;
    while ( *(++ptr) )
    {
        ptr--;
        ch = *ptr;                      //  ch = previous character
        ptr++;
        if ( (*ptr == ',') && (ch == ' ') )
            *ptr = ' ';
    }

    if ( sign_leading )                 //  insert leading sign
    {
        if ( (negative) || (sign == '+') )
        {
            if ( *dest == '0' )         //  leading zeros
            {
                *dest = sign;
            }
            else
            {
                if ( *dest != ' ' )
                {
                    StrFill( dest, '*', strlen(dest) );
                    return( ERROR );
                }
                ptr = dest;
                while ( (*ptr) && (*ptr == ' ') )
                    ptr++;
                ptr--;                  //  backup to first non-blank
                *(ptr--) = sign;        //  backup to first blank
            }
        }
    }
    return( OK );                       //  we made it!
}


#if 0
static  void testmain( INT32 num )
{
    INT16   retcode;
    UCHAR   pic[15];

    fprintf( stdout, "\n" );
    st_cpy( pic, "zzZzzzZzzz.zz+", 15 );        testpic( num, pic );
    st_cpy( pic, "zzZzzzZzzz.zz-", 15 );        testpic( num, pic );
    st_cpy( pic, "9999999999.99+", 15 );        testpic( num, pic );
    st_cpy( pic, "zz,zzz,zz9.99-", 15 );        testpic( num, pic );
    st_cpy( pic, "zz,zzz,zz9.99+", 15 );        testpic( num, pic );
    st_cpy( pic, "-zz,zzz,zz9.99", 15 );        testpic( num, pic );
    st_cpy( pic, "+zz,zzz,zz9.99", 15 );        testpic( num, pic );
    st_cpy( pic, "Zzz,zzz,zz9.99", 15 );        testpic( num, pic );
    st_cpy( pic, "z,zz9.99", 15 );              testpic( num, pic );
    st_cpy( pic, "9,999.99", 15 );              testpic( num, pic );
    st_cpy( pic, "99999.99", 15 );              testpic( num, pic );
    st_cpy( pic, "zzzzz.99", 15 );              testpic( num, pic );
    st_cpy( pic, "zzzzz.zz", 15 );              testpic( num, pic );
    st_cpy( pic, "9.999999", 15 );              testpic( num, pic );
    st_cpy( pic, "zzzzzzzz", 15 );              testpic( num, pic );
    st_cpy( pic, "________", 15 );              testpic( num, pic );
    st_cpy( pic, "99999999", 15 );              testpic( num, pic );
    st_cpy( pic, "99999zzz", 15 );              testpic( num, pic );
    st_cpy( pic, "zzzz9.9",  15 );              testpic( num, pic );
}
static  void testpic( INT32 num, UCHAR *pic )
{
    INT16   retcode;
    UCHAR   string[15];

    retcode = PicAmount( num, string, pic );
    fprintf( stdout, "\n PicAmount <%12ld> = <%s> <%s>", num, string, pic );
    if ( retcode == ERROR )
        fprintf( stdout, " ... ERROR " );
}
main()
{
    INT32   num = 1L;

    fprintf( stdout, "\n\n Format a 32-bit integer into a string according to formatting" );
    fprintf( stdout,   "\n rules contained in a picture, somewhat like COBOL.\n\n" );

    testpic( num, "abc" );
    testpic( num, "-99.9+" );
    num = 100000000L;     testmain( num );
    num =   INT32MIN;     testmain( num );
    num =   INT32MAX;     testmain( num );
    num =         0L;     testmain( num );

doit:
    fprintf( stdout, "\n\n Amount (-9999=quit): " );
    scanf( " %D", &num );
    testmain( num );
    if ( num == -9999L )
        exit(0);
    goto doit;
}
#endif
